@@ -94,7 +94,7 @@ static void check_str_update(struct thread_data *td)
 /*
  * Convert seconds to a printable string.
  */
-static void eta_to_str(char *str, unsigned long eta_sec)
+void eta_to_str(char *str, unsigned long eta_sec)
 {
 	unsigned int d, h, m, s;
 	int disp_hour = 0;
@@ -127,8 +127,10 @@ static int thread_eta(struct thread_data *td)
 	unsigned long long bytes_total, bytes_done;
 	unsigned long eta_sec = 0;
 	unsigned long elapsed;
+	uint64_t timeout;
 
 	elapsed = (mtime_since_now(&td->epoch) + 999) / 1000;
+	timeout = td->o.timeout / 1000000UL;
 
 	bytes_total = td->total_io_size;
 
@@ -174,7 +176,7 @@ static int thread_eta(struct thread_data *td)
 			perc = 1.0;
 
 		if (td->o.time_based) {
-			perc_t = (double) elapsed / (double) td->o.timeout;
+			perc_t = (double) elapsed / (double) timeout;
 			if (perc_t < perc)
 				perc = perc_t;
 		}
@@ -182,10 +184,11 @@ static int thread_eta(struct thread_data *td)
 		eta_sec = (unsigned long) (elapsed * (1.0 / perc)) - elapsed;
 
 		if (td->o.timeout &&
-		    eta_sec > (td->o.timeout + done_secs - elapsed))
-			eta_sec = td->o.timeout + done_secs - elapsed;
+		    eta_sec > (timeout + done_secs - elapsed))
+			eta_sec = timeout + done_secs - elapsed;
 	} else if (td->runstate == TD_NOT_CREATED || td->runstate == TD_CREATED
 			|| td->runstate == TD_INITIALIZED
+			|| td->runstate == TD_SETTING_UP
 			|| td->runstate == TD_RAMP
 			|| td->runstate == TD_PRE_READING) {
 		int t_eta = 0, r_eta = 0;
@@ -196,8 +199,12 @@ static int thread_eta(struct thread_data *td)
 		 * if given, otherwise assume it'll run at the specified rate.
 		 */
 		if (td->o.timeout) {
-			t_eta = td->o.timeout + td->o.start_delay +
-					td->o.ramp_time;
+			uint64_t timeout = td->o.timeout;
+			uint64_t start_delay = td->o.start_delay;
+			uint64_t ramp_time = td->o.ramp_time;
+
+			t_eta = timeout + start_delay + ramp_time;
+			t_eta /= 1000000ULL;
 
 			if (in_ramp_time(td)) {
 				unsigned long ramp_left;
@@ -211,7 +218,7 @@ static int thread_eta(struct thread_data *td)
 		rate_bytes = ddir_rw_sum(td->o.rate);
 		if (rate_bytes) {
 			r_eta = (bytes_total / 1024) / rate_bytes;
-			r_eta += td->o.start_delay;
+			r_eta += (td->o.start_delay / 1000000ULL);
 		}
 
 		if (r_eta && t_eta)
@@ -319,6 +326,7 @@ int calc_thread_status(struct jobs_eta *je, int force)
 		unified_rw_rep += td->o.unified_rw_rep;
 		if (is_power_of_2(td->o.kb_base))
 			je->is_pow2 = 1;
+		je->unit_base = td->o.unit_base;
 		if (td->o.bw_avg_time < bw_avg_time)
 			bw_avg_time = td->o.bw_avg_time;
 		if (td->runstate == TD_RUNNING || td->runstate == TD_VERIFYING
@@ -326,31 +334,32 @@ int calc_thread_status(struct jobs_eta *je, int force)
 		    || td->runstate == TD_PRE_READING) {
 			je->nr_running++;
 			if (td_read(td)) {
-				je->t_rate += td->o.rate[DDIR_READ];
-				je->t_iops += td->o.rate_iops[DDIR_READ];
-				je->m_rate += td->o.ratemin[DDIR_READ];
-				je->m_iops += td->o.rate_iops_min[DDIR_READ];
+				je->t_rate[0] += td->o.rate[DDIR_READ];
+				je->t_iops[0] += td->o.rate_iops[DDIR_READ];
+				je->m_rate[0] += td->o.ratemin[DDIR_READ];
+				je->m_iops[0] += td->o.rate_iops_min[DDIR_READ];
 			}
 			if (td_write(td)) {
-				je->t_rate += td->o.rate[DDIR_WRITE];
-				je->t_iops += td->o.rate_iops[DDIR_WRITE];
-				je->m_rate += td->o.ratemin[DDIR_WRITE];
-				je->m_iops += td->o.rate_iops_min[DDIR_WRITE];
+				je->t_rate[1] += td->o.rate[DDIR_WRITE];
+				je->t_iops[1] += td->o.rate_iops[DDIR_WRITE];
+				je->m_rate[1] += td->o.ratemin[DDIR_WRITE];
+				je->m_iops[1] += td->o.rate_iops_min[DDIR_WRITE];
 			}
 			if (td_trim(td)) {
-				je->t_rate += td->o.rate[DDIR_TRIM];
-				je->t_iops += td->o.rate_iops[DDIR_TRIM];
-				je->m_rate += td->o.ratemin[DDIR_TRIM];
-				je->m_iops += td->o.rate_iops_min[DDIR_TRIM];
+				je->t_rate[2] += td->o.rate[DDIR_TRIM];
+				je->t_iops[2] += td->o.rate_iops[DDIR_TRIM];
+				je->m_rate[2] += td->o.ratemin[DDIR_TRIM];
+				je->m_iops[2] += td->o.rate_iops_min[DDIR_TRIM];
 			}
 
 			je->files_open += td->nr_open_files;
 		} else if (td->runstate == TD_RAMP) {
 			je->nr_running++;
 			je->nr_ramp++;
-		} else if (td->runstate == TD_SETTING_UP)
+		} else if (td->runstate == TD_SETTING_UP) {
 			je->nr_running++;
-		else if (td->runstate < TD_RUNNING)
+			je->nr_setting_up++;
+		} else if (td->runstate < TD_RUNNING)
 			je->nr_pending++;
 
 		if (je->elapsed_sec >= 3)
@@ -360,7 +369,7 @@ int calc_thread_status(struct jobs_eta *je, int force)
 
 		check_str_update(td);
 
-		if (td->runstate > TD_RAMP) {
+		if (td->runstate > TD_SETTING_UP) {
 			int ddir;
 
 			for (ddir = DDIR_READ; ddir < DDIR_RWDIR_CNT; ddir++) {
@@ -446,16 +455,19 @@ void display_thread_status(struct jobs_eta *je)
 	}
 
 	p += sprintf(p, "Jobs: %d (f=%d)", je->nr_running, je->files_open);
-	if (je->m_rate || je->t_rate) {
+	if (je->m_rate[0] || je->m_rate[1] || je->t_rate[0] || je->t_rate[1]) {
 		char *tr, *mr;
 
-		mr = num2str(je->m_rate, 4, 0, je->is_pow2);
-		tr = num2str(je->t_rate, 4, 0, je->is_pow2);
+		mr = num2str(je->m_rate[0] + je->m_rate[1], 4, 0, je->is_pow2, 8);
+		tr = num2str(je->t_rate[0] + je->t_rate[1], 4, 0, je->is_pow2, 8);
 		p += sprintf(p, ", CR=%s/%s KB/s", tr, mr);
 		free(tr);
 		free(mr);
-	} else if (je->m_iops || je->t_iops)
-		p += sprintf(p, ", CR=%d/%d IOPS", je->t_iops, je->m_iops);
+	} else if (je->m_iops[0] || je->m_iops[1] || je->t_iops[0] || je->t_iops[1]) {
+		p += sprintf(p, ", CR=%d/%d IOPS",
+					je->t_iops[0] + je->t_iops[1],
+					je->m_iops[0] + je->m_iops[1]);
+	}
 	if (je->eta_sec != INT_MAX && je->nr_running) {
 		char perc_str[32];
 		char *iops_str[DDIR_RWDIR_CNT];
@@ -467,15 +479,20 @@ void display_thread_status(struct jobs_eta *je)
 		if ((!je->eta_sec && !eta_good) || je->nr_ramp == je->nr_running)
 			strcpy(perc_str, "-.-% done");
 		else {
+			double mult = 100.0;
+
+			if (je->nr_setting_up && je->nr_running)
+				mult *= (1.0 - (double) je->nr_setting_up / (double) je->nr_running);
+
 			eta_good = 1;
-			perc *= 100.0;
+			perc *= mult;
 			sprintf(perc_str, "%3.1f%% done", perc);
 		}
 
 		for (ddir = DDIR_READ; ddir < DDIR_RWDIR_CNT; ddir++) {
 			rate_str[ddir] = num2str(je->rate[ddir], 5,
-						1024, je->is_pow2);
-			iops_str[ddir] = num2str(je->iops[ddir], 4, 1, 0);
+						1024, je->is_pow2, je->unit_base);
+			iops_str[ddir] = num2str(je->iops[ddir], 4, 1, 0, 0);
 		}
 
 		left = sizeof(output) - (p - output) - 1;
